package gg.malloc.defense.ui;

import java.util.Collection;
import java.util.ArrayList;
import java.util.HashMap;

import gg.malloc.defense.model.State;

import org.bukkit.ChatColor;
import org.bukkit.scoreboard.Scoreboard;
import org.bukkit.scoreboard.DisplaySlot;
import org.bukkit.scoreboard.Score;
import org.bukkit.scoreboard.Team;
import org.bukkit.scoreboard.Objective;
import org.bukkit.entity.Player;

public class Sidebar {
  Scoreboard m_scoreboard;
  Objective m_objective;
  ArrayList<String> m_rowKeys;
  HashMap<Integer, String> m_rows;
  ArrayList<Team> m_teams;
  Player m_player;
  int m_size;

  State m_state;

  public Sidebar(State state, Scoreboard scoreboard, Player player) {
    m_state = state;
    m_player = player;
    m_scoreboard = scoreboard;
    m_objective = m_scoreboard.registerNewObjective("text", "dummy", ChatColor.LIGHT_PURPLE + "" + ChatColor.BOLD + "Malloc Defense");
    m_objective.setDisplaySlot(DisplaySlot.SIDEBAR);
    m_rowKeys = new ArrayList<>();
    m_rows = new HashMap<>();
    m_teams = new ArrayList<>();
    m_rowKeys.add(ChatColor.BLACK + "" + ChatColor.WHITE);
    m_rowKeys.add(ChatColor.GOLD+ "" + ChatColor.WHITE);
    m_rowKeys.add(ChatColor.WHITE+ "" + ChatColor.WHITE);
    m_rowKeys.add(ChatColor.BLACK+ "" + ChatColor.BLACK);
    for(String key : m_rowKeys) {
      Team team = m_scoreboard.registerNewTeam(key);
      team.addEntry(key);
      m_teams.add(team);
    }
  }

  public void update() {
    switch(m_state.getStage()) {
      case Idle:
        m_rows.put(0, ChatColor.LIGHT_PURPLE + "Waiting for players...");
        m_size = 1;
        break;
      case Warmup:
        m_rows.put(0, ChatColor.LIGHT_PURPLE + "Wave " + m_state.waveProgress().value() + " / " + m_state.waveProgress().maximum());
        m_rows.put(1, ChatColor.AQUA + "Get ready!");
        m_rows.put(2, ChatColor.AQUA + "Balance: " + m_state.getPlayerBalance(m_player));
        m_rows.put(3, ChatColor.AQUA + "Coins remaining: " + m_state.coinProgress().remaining());
        m_size = 4;
        break;
      case Countdown:
        m_rows.put(0, ChatColor.LIGHT_PURPLE + "Wave " + m_state.waveProgress().value() + " / " + m_state.waveProgress().maximum());
        m_rows.put(1, ChatColor.GOLD + "Wave incoming!");
        m_rows.put(2, ChatColor.AQUA + "Balance: " + m_state.getPlayerBalance(m_player));
        m_rows.put(3, ChatColor.AQUA + "Coins remaining: " + m_state.coinProgress().remaining());
        m_size = 4;
        break;
      case Playing:
        m_rows.put(0, ChatColor.LIGHT_PURPLE + "Wave " + m_state.waveProgress().value() + " / " + m_state.waveProgress().maximum());
        m_rows.put(1, ChatColor.GREEN + "Mobs remaining: " + m_state.mobProgress().remaining());
        m_rows.put(2, ChatColor.AQUA + "Balance: " + m_state.getPlayerBalance(m_player));
        m_rows.put(3, ChatColor.AQUA + "Coins remaining: " + m_state.coinProgress().remaining());
        m_size = 4;
        break;
      case GameOver:
        m_rows.put(0, ChatColor.RED + "Game over!");
        m_size = 1;
    }

    for(int i = 0; i < Math.min(m_size, m_rowKeys.size()); i++) {
      Score rowScore = m_objective.getScore(m_rowKeys.get(i));
      rowScore.setScore(m_rowKeys.size() - i);
      if (m_rows.containsKey(i)) {
        m_teams.get(i).setPrefix(m_rows.get(i));
      } else {
        m_teams.get(i).setPrefix("");
      }
    }
  }

  public Scoreboard getScoreboard() {
    return m_scoreboard;
  }
}
